import unittest

from prep import number_systems, number_codes


class NumberSystemsWiki(unittest.TestCase):
    def setUp(self):
        self.number_systems_dict = number_systems[0]
        self.number_systems_tuple = number_systems[-1]
        self.number_codes_tuple = number_codes

    def test_number_systems_82(self):
        # get back 82 number systems
        self.assertEqual(len(self.number_systems_dict), 82)
        self.assertEqual(len(self.number_systems_tuple), 82)
        self.assertEqual(len(number_systems), 2)

    def test_number_systems_dict_convection(self):
        for nmd, nmt in zip(self.number_systems_dict, self.number_systems_tuple):
            # keys are the defined ones
            self.assertListEqual(['base', 'system', 'system_name'], list(nmd.keys()))
            # all values are strings
            self.assertEqual(
                True,
                all(type(value) == str for value in nmd.values())
            )
            # tuple type is integer
            self.assertEqual(type(nmt), int)

    def test_number_codes_length(self):
        # get 360 codes
        self.assertEqual(len(self.number_codes_tuple), 360)

    def test_number_codes_work(self):
        chr_s = list(filter(lambda c_: bool(c_), [chr(c) for c in self.number_codes_tuple]))
        self.assertEqual(len(chr_s), 360)


if __name__ == '__main__':
    unittest.main()



from .rec import find_recurring_pattern
from . import assure, find_base

ensure_base_is_standard = assure.ensure_base_is_standard
base_limits = assure.ensure_number_representation_is_within_base_limits
base_find = find_base.find

__all__ = ['find_recurring_pattern', 'ensure_base_is_standard', 'base_limits', 'base_find']



def bin_find(arr, x):
    start = 0
    end = len(arr) - 1

    while start <= end:
        mid = (start + end) // 2
        g = arr[mid]

        if g == x:
            return arr[mid]

        if g < x:
            start = mid + 1
        else:
            end = mid - 1


    return None



print(
    bin_find([10, 20, 30, 40, 50, 60, 70, 80, 90, 100, 110], 60)
)


"""
User types in 3 numbers separated by a space
    number_to_convert base_to_convert_to current_base

if current_base is not provided base 10 is assumed

Runs until user types 'q'
"""
from typing import Tuple, Union

from .__dip import dip_this_func


def extract_numbers(string: str) -> Union[Tuple, None]:
    s_string = string.split(' ')
    number = s_string[0]

    if len(s_string) < 2:
        print(f'Please provide a base to convert {number} to')
        return

    try:

        new_base = int(s_string[1])
        try:
            current_base = int(s_string[2])
        except IndexError:
            current_base = 10

    except ValueError:
        print('Ensure the bases are integers')
        return

    return number, new_base, current_base


if __name__ == '__main__':
    run = True

    while run:
        try:
            user_text = input("Enter 'number new_base current_base': ")

            if user_text == 'q':
                run = False
                continue

            elif user_text == 'h':
                print(__doc__)
                continue

            else:
                numbers = extract_numbers(user_text)

                if type(numbers) == tuple and len(numbers) == 3 and all(numbers):
                    dip_this_func(*numbers)

        except KeyboardInterrupt:
            print("EXITING")
            run = False



"""
Convert to and from a representation
"""

from typing import List

from prep import number_codes


def convert_to_representation(numbers: List[int]) -> str:
    """Represent the numbers using the format of one character per number"""
    return ''.join([chr(number_codes[num]) for num in numbers])


def convert_from_representation(representation: str) -> List[int]:
    """Get a list of numbers used to make the representation"""
    return [number_codes.index(ord(c)) for c in representation]



"""
Inside truth.txt, is an extract of the body tag from https://en.wikipedia.org/wiki/List_of_numeral_systems dated
2022-05-02.
Your task is to extract the standard positional numeral systems. Extract base and the name

Found in a .wikitable below a h3 with a span inside with id id="Standard_positional_numeral_systems"

"""
from typing import Dict, List
import re


def extract_details(text: str) -> List[Dict[str, str]]:
    number_systems = []

    # get the table
    text = text.translate(str.maketrans('', '', '\n'))

    table = re.search(
        r'<h3><span.*?id="Standard_positional_numeral_systems".*?<table class="wikitable">(?P<tbody>.*?)</table>',
        text,
    )

    if 'tbody' in table.groupdict().keys():
        table_body = table.group('tbody')
        """
            shall store retrieved number systems in the form 
            [
                {
                    'base': 2,
                    'system': 'Binary Number',
                    'system_name': 'Binary'
                }
            ]            
        """

        # all rows in the table
        table_rows_itr = re.finditer(r'<tr>.*?<td>(?P<base>\d+)</td>.*?<td>(?P<sys>.*?)</td>', table_body)

        for table_row_match in table_rows_itr:
            nms_dict = table_row_match.groupdict()

            if nms_dict:
                trident = {
                    'base': nms_dict.get('base'),
                    'system': nms_dict.get('sys'),
                    'system_name': nms_dict.get('sys')
                }

                if '<a href=' in nms_dict.get('sys'):
                    system_x = re.search(r'title="(?P<sys>.*?)">(?P<sys_name>\w+)</a>', nms_dict.get('sys'))
                    sys_dict = system_x.groupdict()

                    if sys_dict:
                        trident.update({
                            'system': sys_dict.get('sys').replace(' (page does not exist)', ''),
                            'system_name': sys_dict.get('sys_name')
                        })

                number_systems.append(trident)

    return number_systems


with open('truth.txt', 'r', encoding='utf-8') as file:
    details = extract_details(file.read())

    with open('systems.csv', 'w') as w_file:
        title = 'base,system,system_name'
        print(title, file=w_file)
        lines = '\n'.join(
            map(lambda detail: f"{detail.get('base')},{detail.get('system')},{detail.get('system_name')}", details)
        )
        print(lines, file=w_file)



"""
    Converts a list of numbers from one base to another
    the numbers are comma separated
    to use a file as input leave the first argument as 0 or anything and use the marker -if=input_file
    to output to a file use -f=output_file or --file=output_file

    File format of the input file is: e.g., numbers.txt
    1,23,45,6,7,87,86,46,35,24,234,53,6,63,3535

    the numbers are comma separated new lines are assumed to also be separators
"""
import argparse

import n_utils
from conversions import convert_number


class ListArgAction(argparse.Action):
    def __init__(self, option_strings, dest, nargs=None, **kwargs):
        if nargs is not None:
            raise ValueError("nargs not allowed")
        super(ListArgAction, self).__init__(option_strings, dest, **kwargs)

    def __call__(self, parser, namespace, values, option_string=None):
        values = [value for value in values.split(',') if value]
        setattr(namespace, self.dest, values)


par = argparse.ArgumentParser()

par.add_argument(
    'numbers',
    type=str,
    action=ListArgAction,
    help='A list of comma separated numbers in the same base. '
         'To use input file leave this as 0 or anything and add -if'
)

par.add_argument(
    'new_base',
    type=int,
    help='The base to convert the numbers to'
)

par.add_argument(
    '--current_base',
    '-cb',
    type=int,
    default=10,
    help='The current base of the numbers in the list'
)

par.add_argument(
    '--file',
    '-f',
    type=str,
    help='File to save the result in'
)

par.add_argument(
    '--in_file',
    '-if',
    type=str,
    help='File with input, ignores numbers, use 0 instead'
)

args = par.parse_args()

if args.in_file:
    with open(args.in_file, 'r', encoding='utf-8') as in_file:
        text = in_file.read()
        text = text.replace('\n', ',')
        numbers = [number.strip() for number in text.split(',') if number.strip()]
else:
    numbers = args.numbers

current_base = args.current_base
new_base = args.new_base
center_len = 4


# the real work begins
if all([n_utils.base_limits(num, current_base) for num in numbers]) and n_utils.ensure_base_is_standard(new_base):
    current_base_name = n_utils.base_find(current_base)
    new_base_name = n_utils.base_find(new_base)
    converted_numbers = [convert_number(num, current_base, new_base) for num in numbers]
    magic_key = key = lambda item: len(item)
    col_1_len = len(max([current_base_name, max(numbers, key=magic_key)], key=magic_key))
    col_2_len = len(max([new_base_name, max(converted_numbers, key=magic_key)], key=magic_key))
    lone_string = '-' * (col_1_len + 1 + col_2_len + (center_len * 2) + 2)

    numbers_string = f'\n'.join([
        f"|{num.center((col_1_len + center_len))}|"
        f"{conv.center((col_2_len + center_len))}|"
        for num, conv in zip(numbers, converted_numbers)
    ])

    result_string = f"|{current_base_name.center(col_1_len + center_len)}|" \
        f"{new_base_name.center(col_2_len + center_len)}|"

    top_s = f'Base {current_base} to base {new_base}'
    f_string = f'{lone_string}\n{result_string}\n{lone_string}\n{numbers_string}\n{lone_string}'
    final_string = f'{top_s}\n{f_string}'

    if args.file:
        with open(args.file, 'w', encoding='utf-8') as f:
            print(final_string, file=f)

        print(f"Results written to '{args.file}'")
    else:
        print(final_string)


else:
    print("Ensure your arguments are correct: "
          "The base should be standard or the numbers to be within the limit of the current base.")



from functools import lru_cache
from typing import Union

from prep import number_systems


@lru_cache(1024)
def find(base: int) -> Union[str, None]:
    """Given the numeral of a base, find its name"""
    # uses binary search to find the base name
    systems = number_systems[0]
    start = 0
    end = 81

    while start <= end:
        mid = (start + end) // 2
        temp_base = systems[mid]
        temp_base_number = int(temp_base['base'])

        if temp_base_number == base:
            return temp_base['system_name']

        elif temp_base_number < base:
            start = mid + 1

        else:
            end = mid - 1

    return None



from conversions import convert as __convert

__convert_whole_to_base_10 = __convert.convert_to_base_10
__convert_float_to_base_10 = __convert.convert_float_to_base_10
__float_from_base_10 = __convert.convert_float_from_base_10
__whole_from_base_10 = __convert.convert_from_base_10


def convert_number(number: str, base_from: int, base_to: int) -> str:
    """Convert a number from a base to another"""

    if type(number) != str or type(base_from) != int or type(base_to) != int:
        raise TypeError('Enter the correct type arguments')

    # divide up the number by decimal place/ index 0 is the whole number while index -1 is the floating part
    numbers = number.split('.')
    no_float = False

    if len(numbers) == 1:
        numbers.append('0')
        no_float = True

    # convert every thing to base 10
    if base_from == 10:
        base_10_numbers = [int(numbers[0]), float(f'0.{numbers[-1]}')]

    else:
        base_10_numbers = [
            __convert_whole_to_base_10(numbers[0], base_from),
            __convert_float_to_base_10(f'0.{numbers[-1]}', base_from)
        ]

    # convert to new base
    if base_to == 10:
        new_base_numbers = [str(base_10_numbers[0]), str(base_10_numbers[-1])]

    else:
        new_base_numbers = [
            __whole_from_base_10(base_10_numbers[0], base_to)[0],
            __float_from_base_10(base_10_numbers[-1], base_to)[0]
        ]

    # create the final number
    final_number = new_base_numbers[0]

    if not no_float:
        final_number = f'{final_number}.{new_base_numbers[-1].replace("0.", "")}'

    return final_number



import n_utils
from conversions import convert_number


def dip_this_func(number: str, new_base: int, current_base: int):
    """Now the real work begins (read in Ultron's voice)"""
    current_base_name = n_utils.base_find(current_base)
    new_base_name = n_utils.base_find(new_base)

    # ensure the bases to switch between are standard
    if n_utils.ensure_base_is_standard(new_base) and n_utils.ensure_base_is_standard(current_base):

        # Ensure a number is representable within it's base and overall
        if not n_utils.base_limits(number, current_base):
            print(f"'{number}' is not a {current_base_name}")
            return

        result = convert_number(number, current_base, new_base)
        q_string = f"'{number}' in {current_base_name} (base {current_base}) to {new_base_name} (base {new_base}) = "
        print(f"{q_string}'{result}'")

    else:
        non_standard = [
            str(base_) for base_ in [new_base, current_base] if not n_utils.ensure_base_is_standard(base_)
        ]
        print(f"The bases provided are not standard: {', '.join(non_standard)}")



from typing import List
from re import compile as re_compile, search as re_search

from conversions.representations import convert_to_representation, convert_from_representation
from n_utils import find_recurring_pattern


def convert_from_base_10(decimal: int, new_base: int) -> List[str]:
    """
    Takes a decimal and converts it to the the designated number into the new base.
    """
    rems = []
    value = decimal

    while value >= new_base:
        rems.append(value % new_base)
        value //= new_base

    rems.append(value)

    return [convert_to_representation([rem for rem in reversed(rems)]), new_base]


def convert_float_from_base_10(floating_decimal: float, new_base: int) -> List[str]:
    """
    Takes the decimal part and converts it to the new base
    Maximum possible decimal points. However in case of recurring decimal points, the max is 15
    """
    assert 1 > floating_decimal >= 0

    float_match = re_compile(r'\.[0]+$')
    number = floating_decimal
    full_numbers = []

    # multiply until the floating part is zero
    while not re_search(float_match, str(number)):
        number *= new_base
        ex_numbers = str(number).split('.')
        full_numbers.append(ex_numbers[0])
        number = float(f'0.{ex_numbers[-1]}')

        # stop if you find a recurring number no rounding off, just truncate
        if len(full_numbers) > 14 and find_recurring_pattern(''.join(full_numbers)):
            break

    result = convert_to_representation(
        [int(num) for num in full_numbers]
    )

    return [f'0.{result}', new_base]


def convert_to_base_10(number: str, current_base: int) -> int:
    """Takes a number in any base and returns its decimal/base 10 equivalent"""
    numbers = convert_from_representation(number)

    return sum(
        [(current_base**i) * num for i, num in enumerate(reversed(numbers))]
    )


def convert_float_to_base_10(number: str, current_base: int) -> float:
    """Takes a floating number in any base and returns its decimal/base 10 equivalent"""
    assert '.' in number
    number = number.split('.')[-1]
    numbers = convert_from_representation(number)

    return sum(
        [(current_base**-i) * num for i, num in enumerate(numbers, 1)]
    )



import unittest

import n_utils


class UtilsCase(unittest.TestCase):
    def test_recurring_patterns(self):
        self.assertEqual('4', n_utils.find_recurring_pattern('444444'))
        self.assertEqual('geeksforgeeks', n_utils.find_recurring_pattern('geeksforgeeksgeeksforgeeksgeeksforgeeks'))
        self.assertEqual('124', n_utils.find_recurring_pattern('1232124124124124'))
        self.assertEqual('muremwav', n_utils.find_recurring_pattern('mynameismuremwavmuremwavmuremwavmuremwa'))
        self.assertEqual('3', n_utils.find_recurring_pattern('343333333'))

    def test_assurances_standard_base(self):
        self.assertTrue(n_utils.ensure_base_is_standard(2))
        self.assertTrue(n_utils.ensure_base_is_standard(23))
        self.assertTrue(n_utils.ensure_base_is_standard(64))
        self.assertTrue(n_utils.ensure_base_is_standard(8))
        self.assertTrue(n_utils.ensure_base_is_standard(10))
        self.assertTrue(n_utils.ensure_base_is_standard(16))
        self.assertTrue(n_utils.ensure_base_is_standard(216))
        self.assertTrue(n_utils.ensure_base_is_standard(360))

        self.assertFalse(n_utils.ensure_base_is_standard(259))
        self.assertFalse(n_utils.ensure_base_is_standard(63))
        self.assertFalse(n_utils.ensure_base_is_standard(71))
        self.assertFalse(n_utils.ensure_base_is_standard(101))
        self.assertFalse(n_utils.ensure_base_is_standard(301))
        self.assertFalse(n_utils.ensure_base_is_standard(214))
        self.assertFalse(n_utils.ensure_base_is_standard(359))

    def test_base_limits(self):
        self.assertTrue(n_utils.base_limits('AE', 16))
        self.assertTrue(n_utils.base_limits('AG', 30))
        self.assertTrue(n_utils.base_limits('AB', 12))
        self.assertTrue(n_utils.base_limits('10', 10))

        self.assertFalse(n_utils.base_limits('G4', 16))
        self.assertFalse(n_utils.base_limits('G4', 10))
        self.assertFalse(n_utils.base_limits('02', 2))
        self.assertFalse(n_utils.base_limits('9', 8))

    def test_base_find(self):
        self.assertEqual('Undecimal', n_utils.base_find(11))
        self.assertEqual('Decimal', n_utils.base_find(10))
        self.assertEqual('Binary', n_utils.base_find(2))
        self.assertEqual('Trecentosexagesimal', n_utils.base_find(360))
        self.assertEqual('Enneaoctogesimal', n_utils.base_find(89))
        self.assertEqual('Trigesimal', n_utils.base_find(30))


if __name__ == '__main__':
    unittest.main()



from re import compile as re_compile, findall as re_findall


rec_pattern = re_compile(r'(.+?)\1{2}')


def find_recurring_pattern(r_str: str) -> str:
    """
        finds the recurring pattern in a string.
        The pattern has to repeat more that 3 times
        returns the recurring pattern
    """
    res = ''

    c_str = re_findall(rec_pattern, r_str)

    if len(c_str) > 0:
        res = c_str[-1]

    return res



"""Assurances for the numbers presented by user"""
from prep import number_systems, number_codes


def ensure_base_is_standard(base: int) -> bool:
    """
        Ensures the base entered is one of the 82 standard bases.
        Described 'prep/systems.csv' or https://en.wikipedia.org/wiki/List_of_numeral_systems
    """
    return base in number_systems[-1]


def ensure_number_representation_is_within_base_limits(number: str, base: int) -> bool:
    """
        Ensure the representations of a number are within the base character representations
        E.g., G4 cannot be base 16 as G is well beyond base 16
    """
    # . is removed since it's not part of the representation
    # uses the number codes up to a certain place to ensure only character usable for that base are checked
    return all([ord(c) in number_codes[:base] for c in number.replace('.', '')])



"""
Read digits from a file and convert them to expected base
File format is: e.g., numbers.txt
1,23,45,6,7,87,86,46,35,24,234,53,6,63,3535
2
10

first line is the numbers to convert, comma separated
second line is the base to convert to
third line is the current base, if none it's assumed base 10

"""
import argparse
from typing import Union, Tuple

from conversions import convert_number
import n_utils


par = argparse.ArgumentParser()

par.add_argument(
    'input_file',
    type=str,
    help='The path of the file where the numbers are'
)

par.add_argument(
    'output_file',
    type=str,
    help='File to write the results in'
)

args = par.parse_args()


# The real work begins
def extract_details(text: str) -> Union[Tuple, None]:
    split_text = text.split('\n')
    assert len(split_text) in (2, 3)
    detail_numbers = [item.strip() for item in split_text[0].split(',') if item.strip()]

    try:
        detail_new_base = int(split_text[1])

        try:
            detail_current_base = int(split_text[2])
        except IndexError:
            detail_current_base = 10
    except ValueError:
        print("Ensure the bases are integers")
        return

    return detail_numbers, detail_new_base, detail_current_base


with open(args.output_file, 'w', encoding='utf-8') as out_file:
    with open(args.input_file, 'r', encoding='utf-8') as in_file:
        read_text = in_file.read()
        details = extract_details(read_text)

    if type(details) == tuple and len(details) == 3 and all(details):
        numbers = details[0]
        new_base = details[1]
        current_base = details[2]

        if all([n_utils.base_limits(num, current_base) for num in numbers]):
            if n_utils.ensure_base_is_standard(new_base):
                converted_numbers = [convert_number(number, current_base, new_base) for number in numbers]

                print(f'Base {current_base},base {new_base}', file=out_file)
                print(f'{n_utils.base_find(current_base)},{n_utils.base_find(new_base)}', file=out_file)
                print('\n'.join([
                    f'{num},{conv}' for num, conv in zip(numbers, converted_numbers)
                ]), file=out_file)

                print(f"Written to '{args.output_file}'")

            else:
                print(f"Base {new_base} is not standard. Refer to wikipedia")

        else:
            print(f"The numbers are not really base {current_base}")

    else:
        print(f"Format '{args.input_file}' properly")



import unittest

from conversions import convert, representations, master


class ConversionTestCase(unittest.TestCase):

    def setUp(self):
        self.convert_from_base_10 = convert.convert_from_base_10
        self.convert_to_base_10 = convert.convert_to_base_10
        self.convert_from_base_10_float = convert.convert_float_from_base_10
        self.convert_to_base_10_float = convert.convert_float_to_base_10
        self.represent = representations.convert_to_representation
        self.un_represent = representations.convert_from_representation
        self.master = master.convert_number

    def test_conversion_from(self):
        self.assertEqual([1, 14], self.un_represent('1E'))
        self.assertEqual([1, 1, 0], self.un_represent('110'))
        self.assertEqual([1, 7, 2], self.un_represent('172'))
        self.assertEqual([2, 18, 15], self.un_represent('2IF'))

    def test_conversion_to(self):
        self.assertEqual('1E', self.represent([1, 14]))
        self.assertEqual('110', self.represent([1, 1, 0]))
        self.assertEqual('172', self.represent([1, 7, 2]))
        self.assertEqual('2I', self.represent([2, 18]))

    def test_convert_from_base_10(self):
        self.assertRaises(TypeError, self.convert_from_base_10, '10', 2)
        self.assertRaises(TypeError, self.convert_from_base_10, 10, '2')
        self.assertEqual(self.convert_from_base_10(200, 11), ['172', 11])
        self.assertEqual(self.convert_from_base_10(20, 2), ['10100', 2])
        self.assertEqual(self.convert_from_base_10(16, 16), ['10', 16])
        self.assertEqual(self.convert_from_base_10(30, 16), ['1E', 16])

    def test_convert_from_base_10_independent(self):
        self.assertEqual(['1C8', 16], self.convert_from_base_10(456, 16))
        self.assertEqual(['GO', 27], self.convert_from_base_10(456, 27))
        self.assertEqual(['D1', 35], self.convert_from_base_10(456, 35))
        self.assertEqual(['385', 11], self.convert_from_base_10(456, 11))
        self.assertEqual(['DR', 33], self.convert_from_base_10(456, 33))
        self.assertEqual(['E8', 32], self.convert_from_base_10(456, 32))

    def test_convert_from_float_base_10(self):
        self.assertRaises(AssertionError, self.convert_from_base_10_float, 10, 2)
        self.assertRaises(AssertionError, self.convert_from_base_10_float, -1, 2)
        self.assertRaises(TypeError, self.convert_from_base_10_float, '0.2', 2)
        self.assertRaises(TypeError, self.convert_from_base_10_float, 0.2, '2')
        self.assertEqual(['0.AB851EB851EB851E', 16], self.convert_from_base_10_float(0.67, 16))
        self.assertEqual(['0.8', 20], self.convert_from_base_10_float(0.4, 20))
        self.assertEqual(['0.01', 2], self.convert_from_base_10_float(0.25, 2))
        self.assertEqual(['0.444444444444444', 17], self.convert_from_base_10_float(0.25, 17))
        self.assertEqual(['0.2', 8], self.convert_from_base_10_float(0.25, 8))
        self.assertEqual(['0.4', 16], self.convert_from_base_10_float(0.25, 16))
        self.assertEqual(['0.5', 20], self.convert_from_base_10_float(0.25, 20))

    def test_convert_to_base_10(self):
        self.assertRaises(TypeError, self.convert_to_base_10, 23, 5)
        self.assertRaises(TypeError, self.convert_to_base_10, '23', '5')
        self.assertEqual(200, self.convert_to_base_10('172', 11))
        self.assertEqual(20, self.convert_to_base_10('10100', 2))
        self.assertEqual(16, self.convert_to_base_10('10', 16))
        self.assertEqual(30, self.convert_to_base_10('1E', 16))

    def test_convert_to_base_10_independent(self):
        self.assertEqual(456, self.convert_to_base_10('1C8', 16))
        self.assertEqual(456, self.convert_to_base_10('GO', 27))
        self.assertEqual(456, self.convert_to_base_10('D1', 35))
        self.assertEqual(456, self.convert_to_base_10('385', 11))
        self.assertEqual(456, self.convert_to_base_10('DR', 33))
        self.assertEqual(456, self.convert_to_base_10('E8', 32))

    def test_convert_to_base_10_float(self):
        self.assertRaises(TypeError, self.convert_to_base_10_float, 0.3, 4)
        self.assertRaises(TypeError, self.convert_to_base_10_float, '0.3', '4')
        self.assertEqual(0.67, self.convert_to_base_10_float('0.AB851EB851EB851E', 16))
        self.assertEqual(0.4, self.convert_to_base_10_float('0.8', 20))
        self.assertEqual(0.25, self.convert_to_base_10_float('0.01', 2))
        self.assertEqual(0.25, self.convert_to_base_10_float('0.444444444444444', 17))
        self.assertEqual(0.25, self.convert_to_base_10_float('0.2', 8))
        self.assertEqual(0.25, self.convert_to_base_10_float('0.4', 16))
        self.assertEqual(0.25, self.convert_to_base_10_float('0.5', 20))

    def test_master_convert(self):
        self.assertRaises(TypeError, self.master, 12, 10, 2)
        self.assertRaises(TypeError, self.master, '12', '10', 2)
        self.assertRaises(TypeError, self.master, '12', 10, '2')
        self.assertEqual('11001000.001', self.master('C8.2', 16, 2))
        self.assertEqual('C8.2', self.master('11001000.001', 2, 16))
        self.assertEqual('7', self.master('111', 2, 10))


if __name__ == '__main__':
    unittest.main()



import setuptools


with open('README.rst', 'r', encoding='utf-8') as f:
    main_description = f.read()

setuptools.setup(
    name='number-conversions',
    version='0.0.7',
    author='Muremwa',
    author_email='danmburu254@gmail.com',
    url='https://github.com/muremwa/number-conversions',
    description='Convert numbers between different bases.',
    long_description=main_description,
    long_description_content_type='text/x-rst',
    packages=setuptools.find_packages(),
    package_data={'prep': ['*.txt', '*.csv']},
    include_package_data=True,
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires='>=3.6'
)



import argparse

from .__dip import dip_this_func


par = argparse.ArgumentParser()
par.add_argument(
    "number",
    type=str,
    help="Number to convert, (This number is assumed to be in base 10 if --current_base is not provided)"
)
par.add_argument("new_base", type=int, help="Base to convert the number to")
par.add_argument(
    "--current_base",
    "-cb",
    type=int,
    default=10,
    help="Base of the number currently, default is 10 if none is provided"
)
args = par.parse_args()

dip_this_func(args.number, args.new_base, args.current_base)
